/* fixed.h
 *
 * Functions for fixed point arithmetic. (stolen in part from wt :)
 * fixed point format is iiii.ffff where iiii is the integer part
 * and ffff the decimal, in a 32-bit long
 */
#ifndef fixed_h
#define fixed_h

#if FIXED_MATHS

typedef long fixed;

/* fixed point constants */
#define fx1 		65536L
#define fxpi		ftofx(3.1415927)

/* fixed point conversion functions */
#define itofx(i) 	((i) << 16)
#define fxtoi(fx)	((fx) >> 16)
#define fxtof(fx)	(((float)(fx)) / fx1)
#define ftofx(f)	((fixed)((f) * fx1))
#define fxabs(fx)	((fx) < 0 ? -(fx) : (fx))

/* fixed point arithmetic */
#define fxadd(a,b)	((a) + (b))
#define fxsub(a,b)	((a) - (b))
#define fxlow(fx)	((fx) & 0x0000ffff)	/* low 16 bits of fixed */
#define fxhi(fx)	fxtoi(fx)		/* high 16 bits of fixed */
/*
#define fxmul(a,b)	((fxlow(a)*fxlow(b) >> 16) + fxlow(a)*fxhi(b) + \
			fxhi(a)*fxlow(b) + (fxhi(a)*fxhi(b) << 16))
*/
#define fxmul(a,b)	((fxlow(a)*(b) >> 16) + (fxhi(a)*(b)))
#define fxdiv(a,b)	((fixed)(((double)(a) / (double)(b)) * fx1))
#define fxsqrt(fx)	((fixed)(sqrt((double)fxtof(fx)) * fx1))

#else

typedef double fixed;

/* floating constants */
#define fx1		1.0
#define fxpi		3.1415927

/* floating point conversion functions */
#define itofx(i)	((float)(i))
#define fxtoi(fx)	((int)(fx))
#define fxtof(fx)	(fx)
#define ftofx(f)	(f)
#define fxabs(fx)	((fx) < 0 ? -(fx) : (fx))

/* floating point arithmetic */
#define fxadd(a,b)	((a) + (b))
#define fxsub(a,b)	((a) - (b))
#define fxmul(a,b)	((a) * (b))
#define fxdiv(a,b)	((a) / (b))
#define fxsqrt(fx)	((float)sqrt((double)(fx)))

#endif

#endif

